@@ -1,19 +1,19 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
-from hypothesis import settings, strategies as st
+import sys
+import threading
+import warnings
+
+from hypothesis import HealthCheck, given, settings, strategies as st
+
 from tests.common.debug import find_any, minimal
 from tests.common.utils import flaky
 
@@ -127,3 +127,56 @@ def test_can_form_sets_of_recursive_data():
     )
     xs = minimal(trees, lambda x: len(x) >= size, timeout_after=None)
     assert len(xs) == size
+
+
+def test_drawing_from_recursive_strategy_is_thread_safe():
+    shared_strategy = st.recursive(
+        st.integers(), lambda s: st.lists(s, max_size=2), max_leaves=20
+    )
+
+    errors = []
+
+    @settings(
+        database=None, deadline=None, suppress_health_check=[HealthCheck.too_slow]
+    )
+    @given(data=st.data())
+    def test(data):
+        try:
+            # We may get a warning here about not resetting recursionlimit,
+            # since it was changed during execution; ignore it.
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                data.draw(shared_strategy)
+        except Exception as exc:
+            errors.append(exc)
+
+    threads = []
+
+    original_recursionlimit = sys.getrecursionlimit()
+
+    for _ in range(4):
+        threads.append(threading.Thread(target=test))
+
+    for thread in threads:
+        thread.start()
+
+    for thread in threads:
+        thread.join()
+
+    # Cleanup: reset the recursion limit that was (probably) not reset
+    # automatically in the threaded test.
+    sys.setrecursionlimit(original_recursionlimit)
+
+    assert not errors
+
+
+SELF_REF = st.recursive(
+    st.deferred(lambda: st.booleans() | SELF_REF),
+    lambda s: st.lists(s, min_size=1),
+)
+
+
+@given(SELF_REF)
+def test_self_ref_regression(_):
+    # See https://github.com/HypothesisWorks/hypothesis/issues/2794
+    pass
